@@ -283,6 +283,10 @@ void write_pwsafe_file(MainView* main) {
 	if (main->file_name==NULL) {
 		main->file_name=interface_file_chooser(main, GTK_FILE_CHOOSER_ACTION_SAVE);
 	}
+	if (main->file_name==NULL) {
+		//file creation canceled
+		return;
+	}
 	main->file_edited=FALSE;
 	/* init vfs just incase... */
 	gnome_vfs_init();
@@ -403,7 +407,7 @@ void writecbc(MainView* mainview, GnomeVFSHandle *handle, BLOWFISH_CTX *blowCTX,
 gboolean pwsafe_file_open_error(MainView *mainview, GnomeVFSResult vfs_result) {
 	if (vfs_result!=GNOME_VFS_OK) {
 		gchar *txt;
-		txt=g_strjoin(NULL, "Failed to Open '", mainview->file_name, "' (", gnome_vfs_result_to_string(vfs_result), ")", NULL);
+		txt=g_strjoin(NULL, _("Failed to Open"), " '", mainview->file_name, "' (", gnome_vfs_result_to_string(vfs_result), ")", NULL);
 		msgbox(mainview, txt);
 		g_free(txt);
 		mainview->file_name=NULL;
@@ -414,20 +418,20 @@ gboolean pwsafe_file_open_error(MainView *mainview, GnomeVFSResult vfs_result) {
 }
 
 
-void getV1UserTitle(char *buffer, rec *entry) {
+void getV1UserTitle(gchar *buffer, rec *entry) {
 	gchar ** list = g_strsplit(buffer, SEP_V1, 3);
 	if (g_strv_length(list) > 1) {
-		entry->user=g_strndup(list[1], strlen(list[1])-1);
+		entry->user=g_strndup(list[1], strlen(list[1]));
 	} else {
 		entry->user=g_strdup("DefaultUser");
 	}
 	entry->title=g_strdup(list[0]);
 	g_strfreev(list);
-	g_free(buffer);
 }
 
 void read_pwsafe_file(MainView* mainview) {
 	gchar *msg;
+	gchar *title;
 	gboolean valid = FALSE;
 	gboolean v2 = TRUE;
 	/* get password */
@@ -435,20 +439,23 @@ void read_pwsafe_file(MainView* mainview) {
 	g_assert(mainview!=NULL);
 	if (!mainview->file_name) return;
 	file_utf8=g_filename_to_utf8(mainview->file_name, -1, NULL, NULL, NULL);
-	mainview->pass=get_password(mainview, file_utf8, PWD_DLG_GET);
+	title = g_strjoin(NULL, _("Password file : "), g_filename_display_basename(file_utf8), NULL);
+	mainview->pass=get_password(mainview, title, PWD_DLG_GET);
+	g_free(title);
 	g_free(file_utf8);
 	
 	if (mainview->pass && mainview->pass[0]) {
 		GnomeVFSResult vfs_result;
 		GnomeVFSHandle *handle = NULL;
 		GnomeVFSFileInfo finfo;
-		char *buffer=NULL;
+		gchar *buffer=NULL;
 		int buffer_len;
 		unsigned char salt[20];
 		guint32 iv[2];
 		unsigned char rec_type;
-		char *filetitle=NULL;
-		char *fileversion=NULL;
+		gchar *filetitle=NULL;
+		gchar *fileversion=NULL;
+		gchar *titlev1=NULL;
 		gboolean uuiderr=FALSE;
 		SHA1_CTX keyHash;
 		BLOWFISH_CTX blowCTX;
@@ -477,11 +484,13 @@ void read_pwsafe_file(MainView* mainview) {
 							
 							buffer=readcbc(mainview, handle, &blowCTX, iv, &buffer_len, &rec_type);
 							if (buffer!=NULL) {
-								filetitle=buffer;
+								filetitle=g_strndup(buffer, buffer_len);
+								g_free(buffer);
 								buffer=readcbc(mainview, handle, &blowCTX, iv, &buffer_len, &rec_type);
 								if (buffer!=NULL) {
-									fileversion=buffer;
-									if (strcmp("2.0", fileversion)!=0) {
+									fileversion=g_strndup(buffer, buffer_len);
+									g_free(buffer);
+									if (strncmp("2.0", fileversion, 3)!=0) {
 										v2 = FALSE;
 										msg = g_strdup(_("Reading file with version (1.x) \n Will write in 2.0 format"));
 										msgbox(mainview, msg);
@@ -496,11 +505,13 @@ void read_pwsafe_file(MainView* mainview) {
 									recs=NULL;
 									buffer=readcbc(mainview, handle, &blowCTX, iv, &buffer_len, &rec_type);
 									if (v2) {
-										mainview->preferences=buffer;
+										mainview->preferences=g_strndup(buffer, buffer_len);
+										g_free(buffer);
 									} else {
 										if (buffer != NULL) {
-											entry->notes=buffer;
-											entry->group=g_strdup("Group");
+											entry->notes=g_strndup(buffer, buffer_len);
+											g_free(buffer);
+											entry->group=g_strdup(_("Group"));
 											recs=g_list_insert_sorted(recs, entry, (GCompareFunc) rec_comparer);
 										}
 									}
@@ -518,52 +529,66 @@ void read_pwsafe_file(MainView* mainview) {
 													case UUID:
 														if (buffer_len!=16) {
 															if (!uuiderr) {
-																msgbox(mainview, "file error - UUID not 16 bytes. Recreating");
+																msgbox(mainview, _("file error - UUID not 16 bytes. Recreating"));
 															}
 															uuiderr=TRUE;
 															g_free(buffer);
 															new_uuid(entry);
 														} else {
-															entry->uuid=buffer;
+															entry->uuid=g_strndup(buffer, buffer_len);
+															g_free(buffer);
 														}
 														break;
 													case GROUP:
-														entry->group=buffer;
+														entry->group=g_strndup(buffer, buffer_len);
+														g_free(buffer);
 														break;
 													case TITLE:
-														entry->title=buffer;
+														entry->title=g_strndup(buffer, buffer_len);
+														g_free(buffer);
 														break;
 													case USER:
-														entry->user=buffer;
+														entry->user=g_strndup(buffer, buffer_len);
+														g_free(buffer);
 														break;
 													case NOTES:
-														entry->notes=buffer;
+														entry->notes=g_strndup(buffer, buffer_len);
+														g_free(buffer);
 														break;
 													case PASSWORD:
-														entry->password=buffer;
+														entry->password=g_strndup(buffer, buffer_len);
+														g_free(buffer);
 														break;
 													case END:
+														if (!entry->uuid) {
+															new_uuid(entry);
+														}
 														g_free(buffer);
 														recs=g_list_insert_sorted(recs, entry, (GCompareFunc) rec_comparer);
 														entry=g_malloc0(sizeof(rec));
 														entry->uuid=entry->group=entry->title=entry->user=entry->notes=entry->password=NULL;
 														break;
 													default: /* unknown block... just delete it and move on. */
-														if (buffer_len>0) {
+														//if (buffer_len>0) {
 															g_free(buffer);
-														}
+														//}
 												}
 											} else {
 												if (buffer != NULL) {
 													new_uuid(entry);
-													getV1UserTitle(buffer, entry);
+													titlev1 = g_strndup(buffer, buffer_len);
+													g_free(buffer);
+													getV1UserTitle(titlev1, entry);
+													g_free(titlev1);
 													buffer=readcbc(mainview, handle, &blowCTX, iv, &buffer_len, &rec_type);
 													if (buffer != NULL) {
-														entry->password=buffer;
+														entry->password=g_strndup(buffer, buffer_len);
+														g_free(buffer);
 														buffer=readcbc(mainview, handle, &blowCTX, iv, &buffer_len, &rec_type);
 														if (buffer != NULL) {
-															entry->notes=buffer;
-															entry->group=g_strdup("Group");
+															entry->notes=g_strndup(buffer, buffer_len);
+															g_free(buffer);
+															entry->group=g_strdup(_("Group"));
 															recs=g_list_insert_sorted(recs, entry, (GCompareFunc) rec_comparer);
 															entry=g_malloc0(sizeof(rec));
 															entry->uuid=entry->group=entry->title=entry->user=entry->notes=entry->password=NULL;
